#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

#include <check.h>

#include "array_set.h"

/** PUBLIC TESTS **/

START_TEST (sanity_init_clear_size_free)
{
    array_set_t set;
    array_set_init(&set);
    array_set_clear(&set);
    ck_assert(array_set_size(&set) == 0);
    array_set_free(&set);
}
END_TEST

START_TEST (sanity_add_remove_pop_contains)
{
    array_set_t set;
    array_set_init(&set);
    array_set_add(&set, 1);
    array_set_remove(&set, 2);
    ck_assert(array_set_pop(&set) == 1);
    ck_assert(!array_set_contains(&set, 3));
    array_set_free(&set);
}
END_TEST

START_TEST (basic_part1)
{
    array_set_t set;
    array_set_init(&set);
    array_set_add(&set, 1);
    array_set_add(&set, 2);
    array_set_add(&set, 3);
    array_set_remove(&set, 1);
    ck_assert(!array_set_contains(&set, 1));
    ck_assert(array_set_contains(&set, 2));
    data_t v = array_set_pop(&set);
    ck_assert(v == 2 || v == 3);
    ck_assert(array_set_size(&set) == 1);
    array_set_free(&set);
}
END_TEST



/** GENERIC TEST SUITE CODE **/

Suite * test_suite(void)
{
    Suite *s;
    TCase *tc_core;
    s = suite_create("Default");
    tc_core = tcase_create("Core");

    tcase_add_test(tc_core, sanity_init_clear_size_free);
    tcase_add_test(tc_core, sanity_add_remove_pop_contains);
    tcase_add_test(tc_core, basic_part1);


    suite_add_tcase(s, tc_core);
    return s;
}

int run_testsuite()
{
    int number_failed;
    Suite *s;
    SRunner *sr;

    s = test_suite();
    sr = srunner_create(s);

    srunner_run_all(sr, CK_NORMAL);
    number_failed = srunner_ntests_failed(sr);
    srunner_free(sr);

    if (number_failed == 0) {
        printf("SUCCESS: All current tests passed!\n");
    } else {
        printf("FAILURE: At least one test failed or crashed.\n");
    }

    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
}

int main(int argc, char* argv[])
{
    return run_testsuite();
}

